@@ -28,12 +28,13 @@
 (require 'llm)
 (require 'llm-provider-utils)
 (require 'plz-event-source)
+(require 'plz)
 (require 'rx)
 
 ;; Models defined at https://docs.anthropic.com/claude/docs/models-overview
 (cl-defstruct (llm-claude (:include llm-standard-chat-provider))
   (key nil :read-only t)
-  (chat-model "claude-3-5-sonnet-20241022" :read-only t))
+  (chat-model "claude-3-7-sonnet-20250219" :read-only t))
 
 (cl-defmethod llm-nonfree-message-info ((_ llm-claude))
   "Return Claude's nonfree ToS."
@@ -81,13 +82,14 @@
                              (llm-chat-prompt-interactions prompt)))))
         (system (llm-provider-utils-get-system-prompt prompt)))
     (when (llm-chat-prompt-tools prompt)
-      (plist-put request :tools
-                 (vconcat (mapcar (lambda (f) (llm-claude--tool-call f))
-                                  (llm-chat-prompt-tools prompt)))))
+      (setq request (plist-put request :tools
+                               (vconcat
+                                (mapcar (lambda (f) (llm-claude--tool-call f))
+                                        (llm-chat-prompt-tools prompt))))))
     (when (> (length system) 0)
-      (plist-put request :system system))
+      (setq request (plist-put request :system system)))
     (when (llm-chat-prompt-temperature prompt)
-      (plist-put request :temperature (llm-chat-prompt-temperature prompt)))
+      (setq request (plist-put request :temperature (llm-chat-prompt-temperature prompt))))
     (append request (llm-provider-utils-non-standard-params-plist prompt))))
 
 (defun llm-claude--multipart-content (content)
@@ -135,28 +137,86 @@
       (format "Unsupported non-text response: %s" content))))
 
 (cl-defmethod llm-provider-streaming-media-handler ((_ llm-claude)
-                                                    msg-receiver _ err-receiver)
+                                                    receiver err-receiver)
   (cons 'text/event-stream
         (plz-event-source:text/event-stream
          :events `((message_start . ignore)
-                   (content_block_start . ignore)
                    (ping . ignore)
                    (message_stop . ignore)
                    (content_block_stop . ignore)
+                   (message_delta . ignore)
                    (error . ,(lambda (event)
                                (funcall err-receiver (plz-event-source-event-data event))))
+                   (content_block_start
+                    .
+                    ,(lambda (event)
+                       (let* ((data (plz-event-source-event-data event))
+                              (json (json-parse-string data :object-type 'alist))
+                              (block (assoc-default 'content_block json))
+                              (type (assoc-default 'type block))
+                              (index (assoc-default 'index json)))
+                         (when (equal type "tool_use")
+                           (let ((id (assoc-default 'id block))
+                                 (name (assoc-default 'name block)))
+                             (funcall receiver `(:tool-uses-raw
+                                                 ,(vector
+                                                   (list
+                                                    'id id
+                                                    'name name
+                                                    'index index
+                                                    'input "")))))))))
                    (content_block_delta
                     .
                     ,(lambda (event)
                        (let* ((data (plz-event-source-event-data event))
                               (json (json-parse-string data :object-type 'alist))
                               (delta (assoc-default 'delta json))
-                              (type (assoc-default 'type delta)))
-                         (when (equal type "text_delta")
-                           (funcall msg-receiver (assoc-default 'text delta))))))))))
+                              (type (assoc-default 'type delta))
+                              (index (assoc-default 'index json)))
+                         (cond
+                          ((equal type "text_delta")
+                           (funcall receiver `(:text ,(assoc-default 'text delta))))
+                          ((equal type "input_json_delta")
+                           (funcall receiver `(:tool-uses-raw
+                                               ,(vector
+                                                 (list
+                                                  'input (assoc-default 'partial_json delta)
+                                                  'index index)))))))))))))
 
 (cl-defmethod llm-provider-collect-streaming-tool-uses ((_ llm-claude) data)
-  (llm-provider-utils-openai-collect-streaming-tool-uses data))
+  "Transform Claude streaming tool-uses DATA responses into tool use structs.
+DATA is a vector of lists produced by `llm-provider-streaming-media-handler'."
+  (let ((tools (make-hash-table :test 'equal))
+        (index-to-id (make-hash-table :test 'eql))
+        result)
+    (cl-loop for entry across data do
+             (if (plist-get entry 'id)
+                 ;; new tool use
+                 (let ((id (plist-get entry 'id))
+                       (name (plist-get entry 'name))
+                       (index (plist-get entry 'index)))
+                   (puthash id (make-llm-provider-utils-tool-use
+                                :id id :name name :args "")
+                            tools)
+                   (puthash index id index-to-id))
+               ;; tool input update
+               (let* ((index (plist-get entry 'index))
+                      (input (plist-get entry 'input))
+                      (id (gethash index index-to-id))
+                      (tool (gethash id tools)))
+                 (setf (llm-provider-utils-tool-use-args tool)
+                       (concat (llm-provider-utils-tool-use-args tool) input)))))
+    (maphash (lambda (_ tool)
+               (condition-case nil
+                   (let ((args (llm-provider-utils-tool-use-args tool)))
+                     (setf (llm-provider-utils-tool-use-args tool)
+                           (if (string-empty-p args)
+                               nil
+                             (json-parse-string args :object-type 'alist))))
+                 (error nil))
+               (push tool result))
+             tools)
+    (nreverse result)))
 
 (cl-defmethod llm-provider-headers ((provider llm-claude))
   `(("x-api-key" . ,(if (functionp (llm-claude-key provider))
@@ -170,9 +230,13 @@
     (format "Error %s: '%s'" (assoc-default 'type err)
             (assoc-default 'message err))))
 
+(defun llm-claude--url (method)
+  "Return a Claude API URL for METHOD."
+  (format "https://api.anthropic.com/v1/%s" method))
+
 (cl-defmethod llm-provider-chat-url ((_ llm-claude))
   "Return the URL for the Claude API."
-  "https://api.anthropic.com/v1/messages")
+  (llm-claude--url "messages"))
 
 (cl-defmethod llm-chat-token-limit ((provider llm-claude))
   (llm-provider-utils-model-token-limit (llm-claude-chat-model provider)))
@@ -182,7 +246,7 @@
   "Claude")
 
 (cl-defmethod llm-capabilities ((_ llm-claude))
-  (list 'streaming 'function-calls 'image-input 'pdf-input))
+  (list 'streaming 'tool-use 'streaming-tool-use 'image-input 'pdf-input))
 
 (cl-defmethod llm-provider-append-to-prompt ((_ llm-claude) prompt result
                                              &optional tool-use-results)
@@ -193,6 +257,15 @@
                                                                           'user
                                                                         'assistant)))
 
+(cl-defmethod llm-models ((provider llm-claude))
+  (mapcar (lambda (model)
+            (plist-get model :id))
+          (append
+           (plist-get (plz 'get (llm-claude--url "models")
+                        :as (lambda () (json-parse-buffer :object-type 'plist))
+                        :headers (llm-provider-headers provider))
+                      :data)
+           nil)))
 
 (provide 'llm-claude)
 
